@@ -16,4 +16,4 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from .modeling_bart import PipelinedBartForConditionalGeneration
+from .modeling_bart import PipelinedBartForConditionalGeneration, PipelinedBartForSequenceClassification
