@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Optional, Tuple, Union
+
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
@@ -27,7 +29,8 @@ from transformers import (
     BertForSequenceClassification,
     BertForTokenClassification,
 )
-from transformers.models.bert.modeling_bert import BertSelfAttention
+from transformers.modeling_outputs import MaskedLMOutput, QuestionAnsweringModelOutput
+from transformers.models.bert.modeling_bert import BertForPreTrainingOutput, BertSelfAttention
 
 from ...modeling_utils import (
     OnehotGather,
@@ -158,43 +161,65 @@ class PipelinedBertForPreTraining(BertForPreTraining, PipelineMixin):
 
     def forward(
         self,
-        input_ids=None,
-        attention_mask=None,
-        token_type_ids=None,
-        labels=None,
-        next_sentence_label=None,
-    ):
-        output = self.bert(input_ids=input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)
-        sequence_output, pooled_output = output[:2]
+        input_ids: Optional[torch.Tensor] = None,
+        attention_mask: Optional[torch.Tensor] = None,
+        token_type_ids: Optional[torch.Tensor] = None,
+        position_ids: Optional[torch.Tensor] = None,
+        head_mask: Optional[torch.Tensor] = None,
+        inputs_embeds: Optional[torch.Tensor] = None,
+        labels: Optional[torch.Tensor] = None,
+        next_sentence_label: Optional[torch.Tensor] = None,
+        output_attentions: Optional[bool] = None,
+        output_hidden_states: Optional[bool] = None,
+        return_dict: Optional[bool] = None,
+    ) -> Union[Tuple[torch.Tensor], BertForPreTrainingOutput]:
+
+        return_dict = return_dict if return_dict is not None else self.config.use_return_dict
+
+        outputs = self.bert(
+            input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            head_mask=head_mask,
+            inputs_embeds=inputs_embeds,
+            output_attentions=output_attentions,
+            output_hidden_states=output_hidden_states,
+            return_dict=return_dict,
+        )
+        sequence_output, pooled_output = outputs[:2]
 
         if labels is not None:
-            # Select only the masked tokens for the classifier
-            max_number_of_masked_tokens = int(labels.size(1) * 0.25)
-            masked_lm_labels, masked_lm_positions = torch.topk(labels, k=max_number_of_masked_tokens, dim=1)
-            masked_output = self.gather_indices(sequence_output, masked_lm_positions)
-        else:
-            # This case should never happen during training
-            masked_output = sequence_output
+            if hasattr(self.config, "max_num_masked_tokens"):
+                # Select only the masked tokens for the classifier
+                labels, positions = torch.topk(labels, k=self.config.max_num_masked_tokens, dim=1)
+                sequence_output = self.gather_indices(sequence_output, positions)
 
-        prediction_scores, sequential_relationship_score = self.cls(masked_output, pooled_output)
-        output = (
-            prediction_scores,
-            sequential_relationship_score,
-        ) + output[2:]
+        prediction_scores, seq_relationship_score = self.cls(sequence_output, pooled_output)
 
+        total_loss = None
         if labels is not None and next_sentence_label is not None:
             masked_lm_loss = F.cross_entropy(
                 prediction_scores.view(-1, self.config.vocab_size),
-                masked_lm_labels.view(-1),
-                ignore_index=-100,
+                labels.view(-1),
             ).float()
             next_sentence_loss = F.cross_entropy(
-                sequential_relationship_score.view(-1, 2), next_sentence_label.view(-1)
+                seq_relationship_score.view(-1, 2), next_sentence_label.view(-1)
             ).float()
             total_loss = poptorch.identity_loss(masked_lm_loss + next_sentence_loss, reduction="none")
-            return (total_loss, masked_lm_loss, next_sentence_loss)
 
-        return output
+        # If labels are provided (training mode) only output the loss
+        if not return_dict:
+            output = (prediction_scores, seq_relationship_score) + outputs[2:]
+            return (total_loss,) if total_loss is not None else output
+
+        return BertForPreTrainingOutput(
+            loss=total_loss,
+            prediction_logits=prediction_scores if total_loss is None else None,
+            seq_relationship_logits=seq_relationship_score if total_loss is None else None,
+            hidden_states=outputs.hidden_states if total_loss is None else None,
+            attentions=outputs.attentions if total_loss is None else None,
+        )
 
 
 @register(BertForMaskedLM)
@@ -283,31 +308,77 @@ class PipelinedBertForMaskedLM(BertForMaskedLM, PipelineMixin):
             self.tie_weights()
         return self
 
-    def forward(self, input_ids, attention_mask, token_type_ids=None, labels=None):
+    def forward(
+        self,
+        input_ids: Optional[torch.Tensor] = None,
+        attention_mask: Optional[torch.Tensor] = None,
+        token_type_ids: Optional[torch.Tensor] = None,
+        position_ids: Optional[torch.Tensor] = None,
+        head_mask: Optional[torch.Tensor] = None,
+        inputs_embeds: Optional[torch.Tensor] = None,
+        encoder_hidden_states: Optional[torch.Tensor] = None,
+        encoder_attention_mask: Optional[torch.Tensor] = None,
+        labels: Optional[torch.Tensor] = None,
+        output_attentions: Optional[bool] = None,
+        output_hidden_states: Optional[bool] = None,
+        return_dict: Optional[bool] = None,
+    ) -> Union[Tuple[torch.Tensor], MaskedLMOutput]:
+        r"""
+        labels (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
+            Labels for computing the masked language modeling loss. Indices should be in `[-100, 0, ...,
+            config.vocab_size]` (see `input_ids` docstring) Tokens with indices set to `-100` are ignored (masked), the
+            loss is only computed for the tokens with labels in `[0, ..., config.vocab_size]`
+        """
+
+        return_dict = return_dict if return_dict is not None else self.config.use_return_dict
+
         if self.training:
-            output = self.bert(input_ids=input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)
-            sequence_output = output[0]
+            outputs = self.bert(
+                input_ids,
+                attention_mask=attention_mask,
+                token_type_ids=token_type_ids,
+                position_ids=position_ids,
+                head_mask=head_mask,
+                inputs_embeds=inputs_embeds,
+                encoder_hidden_states=encoder_hidden_states,
+                encoder_attention_mask=encoder_attention_mask,
+                output_attentions=output_attentions,
+                output_hidden_states=output_hidden_states,
+                return_dict=return_dict,
+            )
+            sequence_output = outputs[0]
 
-            # Select only the masked tokens for the classifier
-            max_number_of_masked_tokens = int(labels.size(1) * 0.25)
-            masked_lm_labels, masked_lm_positions = torch.topk(labels, k=max_number_of_masked_tokens, dim=1)
-            masked_output = self.gather_indices(sequence_output, masked_lm_positions)
+            if hasattr(self.config, "max_num_masked_tokens"):
+                # Select only the masked tokens for the classifier
+                labels, positions = torch.topk(labels, k=self.config.max_num_masked_tokens, dim=1)
+                sequence_output = self.gather_indices(sequence_output, positions)
 
-            prediction_scores = self.cls(masked_output)
-            output = (prediction_scores,) + output[2:]
+            prediction_scores = self.cls(sequence_output)
+            outputs = (prediction_scores,) + outputs[2:]
 
             masked_lm_loss = F.cross_entropy(
-                prediction_scores.view(-1, self.config.vocab_size), masked_lm_labels.view(-1)
+                prediction_scores.view(-1, self.config.vocab_size), labels.view(-1)
             ).float()
-            return (masked_lm_loss,)
+            # When training only return the loss
+            if return_dict:
+                return MaskedLMOutput(loss=masked_lm_loss)
+            else:
+                return (masked_lm_loss,)
 
         else:
             return super().forward(
                 input_ids=input_ids,
                 attention_mask=attention_mask,
                 token_type_ids=token_type_ids,
+                position_ids=position_ids,
+                head_mask=head_mask,
+                inputs_embeds=inputs_embeds,
+                encoder_hidden_states=encoder_hidden_states,
+                encoder_attention_mask=encoder_attention_mask,
+                output_attentions=output_attentions,
+                output_hidden_states=output_hidden_states,
                 labels=labels,
-                return_dict=False,
+                return_dict=return_dict,
             )
 
 
@@ -383,15 +454,6 @@ class PipelinedBertForSequenceClassification(BertForSequenceClassification, Bert
         logger.info("-----------------------------------------------------------")
         return self
 
-    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
-        return super().forward(
-            input_ids=input_ids,
-            attention_mask=attention_mask,
-            token_type_ids=token_type_ids,
-            labels=labels,
-            return_dict=False,
-        )
-
 
 @register(BertForMultipleChoice)
 class PipelinedBertForMultipleChoice(BertForMultipleChoice, BertPipelineMixin):
@@ -412,15 +474,6 @@ class PipelinedBertForMultipleChoice(BertForMultipleChoice, BertPipelineMixin):
         logger.info("-----------------------------------------------------------")
         return self
 
-    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
-        return super().forward(
-            input_ids=input_ids,
-            attention_mask=attention_mask,
-            token_type_ids=token_type_ids,
-            labels=labels,
-            return_dict=False,
-        )
-
 
 @register(BertForTokenClassification)
 class PipelinedBertForTokenClassification(BertForTokenClassification, BertPipelineMixin):
@@ -441,15 +494,6 @@ class PipelinedBertForTokenClassification(BertForTokenClassification, BertPipeli
         logger.info("-----------------------------------------------------------")
         return self
 
-    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
-        return super().forward(
-            input_ids=input_ids,
-            attention_mask=attention_mask,
-            token_type_ids=token_type_ids,
-            labels=labels,
-            return_dict=False,
-        )
-
 
 @register(BertForQuestionAnswering)
 class PipelinedBertForQuestionAnswering(BertForQuestionAnswering, BertPipelineMixin):
@@ -470,15 +514,46 @@ class PipelinedBertForQuestionAnswering(BertForQuestionAnswering, BertPipelineMi
         logger.info("-----------------------------------------------------------")
         return self
 
-    def forward(self, input_ids, attention_mask, token_type_ids, start_positions=None, end_positions=None):
+    def forward(
+        self,
+        input_ids: Optional[torch.Tensor] = None,
+        attention_mask: Optional[torch.Tensor] = None,
+        token_type_ids: Optional[torch.Tensor] = None,
+        position_ids: Optional[torch.Tensor] = None,
+        head_mask: Optional[torch.Tensor] = None,
+        inputs_embeds: Optional[torch.Tensor] = None,
+        start_positions: Optional[torch.Tensor] = None,
+        end_positions: Optional[torch.Tensor] = None,
+        output_attentions: Optional[bool] = None,
+        output_hidden_states: Optional[bool] = None,
+        return_dict: Optional[bool] = None,
+    ) -> Union[Tuple[torch.Tensor], QuestionAnsweringModelOutput]:
+        r"""
+        start_positions (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
+            Labels for position (index) of the start of the labelled span for computing the token classification loss.
+            Positions are clamped to the length of the sequence (`sequence_length`). Position outside of the sequence
+            are not taken into account for computing the loss.
+        end_positions (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
+            Labels for position (index) of the end of the labelled span for computing the token classification loss.
+            Positions are clamped to the length of the sequence (`sequence_length`). Position outside of the sequence
+            are not taken into account for computing the loss.
+        """
+        return_dict = return_dict if return_dict is not None else self.config.use_return_dict
+
         output = super().forward(
-            input_ids=input_ids,
+            input_ids,
             attention_mask=attention_mask,
             token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            head_mask=head_mask,
+            inputs_embeds=inputs_embeds,
             start_positions=start_positions,
             end_positions=end_positions,
-            return_dict=False,
+            output_attentions=output_attentions,
+            output_hidden_states=output_hidden_states,
+            return_dict=return_dict,
         )
+
         if start_positions is not None and end_positions is not None:
             output = (poptorch.identity_loss(output[0], reduction="none"),) + output[1:]
         return output
